/*
 * Turbo Programmer Utilities, turbo-prog-utils, www.bladox.com 
 *
 * Copyright (C) 2004, 2005 BLADOX, s.r.o.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <config.h>
#include <tprog/tprog.h>
#include <stdlib.h>
#include <avr/interrupt.h>

#include "turboadapter.h"
#include "btldr.h"
#include "dbg.h"
#include "clka.h"

void turbo_handler (u8 action, USB_Data * data)
{
  u8 *buf = buf_A ();
  u16 i;
u8 what;

  switch (action)
  {
    case ACTION_APP_VERSION:
      data->buf[0] = VERSION_A;
      data->buf[1] = VERSION_B;
      data->buf[2] = VERSION_C;
      break;

    case ACTION_TURBO_ADAPTER_BTLDR_INIT:
      reg_action (ACTION_SIG_UART1_RECV);
      init_vars ();
      init_turboadapter_btldr_port (BTLDR_PORT_TURBO_ADAPTER);
      init_btldrUart ();
      init_clka_timer ();
      reset_turbo_for_bootloader (BTLDR_PORT_TURBO_ADAPTER);

      btldrTxSendChar (UCMD_BTLDR_MODE_INIT);
      while (btldrRxChars () == 0);
      buf[0] = btldrRxGetChar ();

      usb_send (NO_ERROR, 1, buf);
      break;

    case ACTION_TURBO_ADAPTER_BTLDR_SIM_INIT:
      reg_action (ACTION_SIG_UART1_RECV);
      init_vars ();
      init_turboadapter_btldr_sim_port ();
      init_btldrUart ();
      init_clka_timer ();
      reset_turbo_for_bootloader_sim ();

      btldrTxSendChar (UCMD_BTLDR_MODE_INIT);
      while (btldrRxChars () == 0);
      buf[0] = btldrRxGetChar ();

      usb_send (NO_ERROR, 1, buf);
      break;

   case ACTION_TURBO_ADAPTER_BTLDR_INIT_V3:
      reg_action (ACTION_SIG_UART1_RECV);
      init_vars ();
	what=data->buf[0];

if(what==BTLDR_PORT_SIM)
		init_turboadapter_btldr_sim_port ();
else
		init_turboadapter_btldr_port (what);

      init_btldrUart ();
      init_clka_timer ();

if(what==BTLDR_PORT_SIM)
      		reset_turbo_for_bootloader_sim ();
else
      		reset_turbo_for_bootloader ();

      btldrTxSendChar (UCMD_BTLDR_MODE_INIT);
      while (btldrRxChars () == 0);
      buf[0] = btldrRxGetChar ();

      usb_send (NO_ERROR, 1, buf);
      break;

    case ACTION_SIG_UART1_RECV:
      btldr_uart_irq ();
      break;

    case ACTION_TURBO_ADAPTER_BTLDR_WRITE_PAGE:
      btldr_flash_page (data, buf);
      break;
    case ACTION_TURBO_ADAPTER_BTLDR_READ_PAGE:
      btldr_read_page (data, buf);
      break;
    case ACTION_TURBO_ADAPTER_BTLDR_WRITE_PAGE_EEPROM:
      btldr_flash_page_eeprom (data, buf);
      break;
    case ACTION_TURBO_ADAPTER_BTLDR_READ_PAGE_EEPROM:
      btldr_read_page_eeprom (data, buf);
      break;

    case ACTION_TURBO_ADAPTER_BTLDR_SRAM_FLASH:
      btldr_sram_flash (data, buf);
      break;
    case ACTION_TURBO_ADAPTER_BTLDR_SRAM_EEPROM:
      btldr_sram_eeprom (data, buf);
      break;

// following is "dbg" related

    case ACTION_TURBO_ADAPTER_DBG_INIT:
      reg_action (ACTION_SIG_SPI);
      init_turboadapter_dbg_port ();
      usb_send (NO_ERROR, 0, NULL);
      break;

    case ACTION_SIG_SPI:
      usb_put_char_nb (inb (SPDR));
#if 0
      {
	u8 x = inb (SPDR);

	usb_put_char_nb (x);
	led_on (x % 8);
      }
#endif
      break;

    default:
      break;
  }
}
